import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { Area } from '../entities/Area';
import { Subarea } from '../entities/Subarea';

class AreaController {
    static async getAreas(req: Request, res: Response): Promise<Response> {
        try {
            const areaRepository = getRepository(Area);
            const areas = await areaRepository.find({ relations: ["unidade", "subareas"], order: { nome: 'ASC' } });
            return res.json(areas);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async getArea(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const areaRepository = getRepository(Area);
            const area = await areaRepository.findOne(id, { relations: ["unidade", "subareas"] });
            if (!area) {
                return res.status(404).json({ message: "Area not found" });
            }
            return res.json(area);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async createArea(req: Request, res: Response): Promise<Response> {
        try {
            const areaRepository = getRepository(Area);
            const area = areaRepository.create(req.body);
            const result = await areaRepository.save(area);
            return res.status(201).json(result);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async updateArea(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const areaRepository = getRepository(Area);
            delete req.body.userId;
            await areaRepository.update(id, req.body);
            const updatedArea = await areaRepository.findOne(id);
            if (!updatedArea) {
                return res.status(404).json({ message: "Area not found" });
            }
            return res.json(updatedArea);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async deleteArea(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const areaRepository = getRepository(Area);
            const result = await areaRepository.delete(id);
            if (result.affected === 0) {
                return res.status(404).json({ message: "Area not found" });
            }
            return res.status(204).send();
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async getSubareas(req: Request, res: Response): Promise<Response> {
        const { id } = req.params;
        try {
            const subareaRepository = getRepository(Subarea);
            const subareas = await subareaRepository.find({ where: { area: id } });
            return res.json(subareas);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

}

export default AreaController;
