import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { Categorias } from '../entities/Categorias';
import { TiposViatura } from '../entities/TiposViatura';

class CategoriaController {
    static async list(req: Request, res: Response): Promise<Response> {
        try {
            const repository = getRepository(Categorias);
            const categorias = await repository.find({ order: { descricao: 'ASC' }, relations: ['tiposViatura'] });
            return res.json(categorias);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Categorias);
            const categoria = await repository.findOne(id, { relations: ['tiposViatura'] });
            if (!categoria) {
                return res.status(404).json({ message: "Categoria não encontrada" });
            }
            return res.json(categoria);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { descricao, comentarios, situacao, tiposViaturaIds } = req.body;
            const repository = getRepository(Categorias);
            const tiposViaturaRepository = getRepository(TiposViatura);

            let existeRegistro = await repository.findOne({ where: { descricao } });
            if (existeRegistro) {
                return res.status(400).json("Já existe uma categoria com esta descrição.");
            }

            const tiposViatura = await tiposViaturaRepository.findByIds(tiposViaturaIds);
            const categoria = repository.create({
                descricao,
                comentarios,
                situacao,
                tiposViatura
            });

            const result = await repository.save(categoria);
            return res.status(200).json(result);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar a categoria. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, comentarios, situacao, tiposViaturaIds } = req.body;
            const repository = getRepository(Categorias);
            const tiposViaturaRepository = getRepository(TiposViatura);

            let existeRegistro = await repository.findOne({ where: { descricao } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe uma categoria com esta descrição.");
            }

            const categoriaToUpdate = await repository.findOne(id, { relations: ['tiposViatura'] });
            if (!categoriaToUpdate) {
                return res.status(404).json("Categoria não encontrada");
            }

            const tiposViatura = await tiposViaturaRepository.findByIds(tiposViaturaIds);

            repository.merge(categoriaToUpdate, {
                descricao,
                comentarios,
                situacao,
                tiposViatura
            });

            const result = await repository.save(categoriaToUpdate);
            return res.json(result);
        } catch (error) {
            return res.status(500).json("Erro ao salvar a categoria. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Categorias);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Categoria não encontrada');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir a categoria. Tente novamente.");
        }
    }

    static async listaPorTipoViatura(req: Request, res: Response): Promise<Response> {
        try {
            const { tipo } = req.params;

            const repository = getRepository(Categorias);

            const categorias = await repository
                .createQueryBuilder("categorias")
                .leftJoinAndSelect("categorias.tiposViatura", "tiposViatura")
                .where("tiposViatura.nome = :tipo", { tipo })
                .orderBy("categorias.descricao", "ASC")
                .getMany();

            return res.json(categorias);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }
}

export default CategoriaController;
