import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { Categorias } from '../entities/Categorias';
import { InventariosMateriais } from '../entities/InventariosMateriais';
import { Inventarios } from '../entities/Inventarios';
import { CheckLists } from '../entities/CheckLists';
import { ItensCheckList } from '../entities/ItensCheckList';
import { User } from '../entities/User';
import { format } from 'date-fns';
import { Viaturas } from '../entities/Viaturas';

const getChecklistsByDate = async (date, userId) => {
    const checklistRepository = getRepository(CheckLists);
    const viaturasRepository = getRepository(Viaturas);

    const repositoryUsuario = getRepository(User);

    const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });

    const unidadesIds = usuario.unidades.map(unidade => unidade.id);

    const viaturas = await viaturasRepository
        .createQueryBuilder("viatura")
        .where('viatura.unidade IN (:...unidadesIds)', { unidadesIds })
        .orderBy("viatura.prefixo", "ASC")
        .addOrderBy("viatura.numero", "ASC")
        .getMany();

    const checklists = await checklistRepository
        .createQueryBuilder("checklist")
        .leftJoinAndSelect("checklist.viatura", "viatura")
        .leftJoinAndSelect("checklist.tipoEquipamento", "tipoEquipamento")
        .where("checklist.data = :date", { date })
        .getMany();

    const viaturaChecklists = viaturas.map(viatura => {
        const viaturaChecklists = checklists.filter(checklist => checklist.viatura.id === viatura.id);
        return {
            viatura,
            checklists: viaturaChecklists,
            hasChecklist: viaturaChecklists.length > 0
        };
    });

    return viaturaChecklists;
}

const getChecklistsByDateWithTypes = async (date) => {
    const checklistRepository = getRepository(CheckLists);
    const viaturasRepository = getRepository(Viaturas);

    const viaturas = await viaturasRepository
        .createQueryBuilder("viatura")
        .leftJoinAndSelect("viatura.unidade", "unidade")
        .leftJoinAndSelect("unidade.area", "area")
        .leftJoinAndSelect("viatura.viaturastatus", "viaturastatus")
        .leftJoinAndSelect("viaturastatus.status", "status")
        .orderBy("viaturastatus.data", "DESC")
        .addOrderBy("viatura.prefixo", "ASC")
        .addOrderBy("viatura.numero", "ASC")
        .getMany();

    const viaturasWithRecentStatus = viaturas.map(viatura => {
        const recentStatus = viatura.viaturastatus.sort((a: any, b: any) => b.data - a.data)[0];
        return {
            ...viatura,
            status: recentStatus ? [recentStatus] : []
        };
    });

    const checklists = await checklistRepository
        .createQueryBuilder("checklist")
        .leftJoinAndSelect("checklist.viatura", "viatura")
        .leftJoinAndSelect("checklist.tipoEquipamento", "tipoEquipamento")
        .where("checklist.data = :date", { date })
        .getMany();

    console.log('Checklists', checklists);

    const viaturaChecklists = viaturas.map(viatura => {
        const viaturaChecklists = checklists.filter(checklist => checklist.viatura.id === viatura.id);
        const checklistTypes = {
            Equipamento: viaturaChecklists.some(checklist => checklist.tipoEquipamento.descricao === 'Equipamento') ? 'Sim' : 'Não',
            Viatura: viaturaChecklists.some(checklist => checklist.tipoEquipamento.descricao === 'Viatura') ? 'Sim' : 'Não',
            Implemento: viaturaChecklists.some(checklist => checklist.tipoEquipamento.descricao === 'Implemento') ? 'Sim' : 'Não',
        };

        return {
            viatura,
            unidade: viatura.unidade,
            viaturastatus: viatura.viaturastatus, // Inclui o status mais recente
            checklists: viaturaChecklists,
            checklistTypes,
            hasChecklist: viaturaChecklists.length > 0
        };
    });

    return viaturaChecklists;
}


class CheckListController {
    static async list(req: Request, res: Response): Promise<Response> {
        const { tipoEquipamento } = req.params;
        try {
            const repository = getRepository(CheckLists);
            const checklists = await repository.find({ where: { tipoEquipamento }, relations: ['viatura', 'itensCheckLists'], order: { id: 'ASC' } })
            return res.json(checklists);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(CheckLists);
            const checklist = await repository.findOne({ where: { id }, relations: ['viatura', 'itensCheckLists'] });
            if (!checklist) {
                return res.status(404).json({ message: "Checklist não encontrado" });
            }
            return res.json(checklist);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { viatura, equipamentos, tipoEquipamento, userId } = req.body;
            const equipamentosObj: { [key: string]: any } = equipamentos;

            const data = new Date();

            const repositoryCheckList = getRepository(CheckLists);
            const repositoryItensCheckList = getRepository(ItensCheckList);

            console.log('Equipamentos', equipamentosObj);

            let existeRegistro = await repositoryCheckList.findOne({ where: { data, viatura, tipoEquipamento } });
            if (existeRegistro) {
                return res.status(400).json("Já existe um checklist para esta viatura hoje.");
            }

            const checklist = new CheckLists();
            checklist.viatura = viatura;
            checklist.tipoEquipamento = tipoEquipamento;
            checklist.data = data;
            checklist.usuario = userId;
            checklist.situacao = 'Rascunho';

            const result = await repositoryCheckList.save(checklist);
            let status = "Sem pendência";

            if (tipoEquipamento == 1) {
                for (const equipamento of Object.values(equipamentosObj)) {
                    for (const material of equipamento.materiais) {
                        const itens = new ItensCheckList();
                        itens.categoria = material.categoria;
                        itens.quantidade = material.quantidadeReportada ?? material.quantidade;
                        itens.equipamento = material.equipamento;
                        itens.inventarioMaterial = material.id;
                        itens.descricaoPendencia = material.descricaoRelatorio ?? null;
                        itens.checklist = result;

                        if (material.quantidadeReportada !== undefined || material.descricaoRelatorio !== undefined) {
                            status = "Possui pendência";
                        }
                        await repositoryItensCheckList.save(itens);
                    }
                }
            } else {
                for (const invMaterial of Object.values(equipamentosObj)) {
                    const itens = new ItensCheckList();
                    itens.equipamento = invMaterial.equipamento;
                    itens.inventarioMaterial = invMaterial.id;
                    itens.descricaoPendencia = invMaterial.descricaoRelatorio;
                    itens.checklist = result;

                    if (invMaterial.descricaoRelatorio !== undefined) {
                        status = "Possui pendência";
                    }

                    await repositoryItensCheckList.save(itens);
                }
            }

            result.situacao = status;
            await repositoryCheckList.save(result);

            return res.status(200).json("Checklist criado.");
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o checklist. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, viatura, equipamentos, situacao, tipoEquipamento, equipamentosRemover } = req.body;
            console.log('Body', req.body);

            const repository = getRepository(Inventarios);
            const repositoryMaterial = getRepository(InventariosMateriais);

            let existeRegistro = await repository.findOne({ where: { descricao, tipoEquipamento } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe um inventário com essa descrição.");
            }

            existeRegistro = await repository.findOne({ where: { viatura, tipoEquipamento } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe um inventário com essa viatura.");
            }

            const inventarioToUpdate = await repository.findOne(id);
            if (!inventarioToUpdate) {
                return res.status(404).json("Inventário não encontrado");
            }



            console.log('Inventario encontrado', inventarioToUpdate);
            console.log('Equipamentos', equipamentos);
            console.log('Equipamentos Remover', equipamentosRemover);

            const data = { descricao, viatura, situacao };

            repository.merge(inventarioToUpdate, data);
            const result = await repository.save(inventarioToUpdate);

            if (tipoEquipamento == 1) {
                for (const e of equipamentos) {
                    if (!e.id) {
                        const inventarioMaterial = new InventariosMateriais();
                        inventarioMaterial.categoria = e.categoria;
                        inventarioMaterial.quantidade = e.quantidade;
                        inventarioMaterial.equipamento = e.equipamento;
                        inventarioMaterial.inventario = inventarioToUpdate;

                        await repositoryMaterial.save(inventarioMaterial);
                    }
                }
            } else {
                for (const e of equipamentos) {
                    if (!e.id) {
                        const inventarioMaterial = new InventariosMateriais();
                        inventarioMaterial.equipamento = e.equipamento;
                        inventarioMaterial.inventario = inventarioToUpdate;

                        await repositoryMaterial.save(inventarioMaterial);
                    }
                }
            }

            for (const r of equipamentosRemover) {
                await repositoryMaterial.softDelete(r.id);
            }

            console.log('Resultado', result);

            return res.json(result);
        } catch (error) {
            console.error(error);
            return res.status(500).json("Erro ao salvar a inventário. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Inventarios);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Inventário não encontrada');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir a inventário. Tente novamente.");
        }
    }

    static async listChecklists(req: Request, res: Response): Promise<Response> {
        try {
            const { userId } = req.body;
            const { data } = req.params;
            console.log('Data param', data)
            const date = data != '0' ? data : new Date();
            console.log('Data', date)
            const checklists = await getChecklistsByDate(date, userId);

            const result = checklists.map(({ viatura, checklists, hasChecklist }) => ({
                viatura: viatura,
                totalChecklists: checklists.length,
                tipoChecklists: checklists.map(checklist => checklist.tipoEquipamento?.descricao || 'Tipo desconhecido'),
                mensagem: hasChecklist ? '' : 'Checklist não enviado'
            }));

            return res.status(200).json(result);
        } catch (error) {
            console.error("Erro ao listar checklists: ", error);
            res.status(500).json({ message: "Erro ao listar checklists" });
        }
    };

    static async listChecklistsWithTypes(req: Request, res: Response): Promise<Response> {
        try {
            const { data } = req.params;
            console.log('Data param', data)
            const date = data != '0' ? data : new Date();
            console.log('Data', date);
            const checklists = await getChecklistsByDateWithTypes(date);

            const result = checklists.map(({ viatura, checklistTypes, hasChecklist }) => ({
                viatura: viatura,
                unidade: viatura.unidade,
                viaturastatus: viatura.viaturastatus,
                checklistTypes,
                mensagem: hasChecklist ? '' : 'Checklist não enviado'
            }));

            return res.status(200).json(result);
        } catch (error) {
            console.error("Erro ao listar checklists: ", error);
            res.status(500).json({ message: "Erro ao listar checklists" });
        }
    }
}

export default CheckListController;