import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import fs from 'fs';
import path from 'path';
import { promisify } from 'util';
import { Imagens } from '../entities/Imagens';
import { where } from 'sequelize';
import { Equipamentos } from '../entities/Equipamentos';
import { UnidadesMateriais } from '../entities/UnidadesMateriais';
import { Viaturas } from '../entities/Viaturas';

class EquipamentoController {

    static async listAll(req: Request, res: Response): Promise<Response> {
        try {
            const repository = getRepository(Equipamentos);
            const equipamentos = await repository.find({ relations: ['tipoEquipamento'], order: { descricao: 'ASC' } })
            return res.json(equipamentos);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async list(req: Request, res: Response): Promise<Response> {
        const { tipo_equipamento_id } = req.params;
        try {
            const repository = getRepository(Equipamentos);
            const equipamentos = await repository.find({ where: { tipoEquipamento: tipo_equipamento_id }, relations: ['tipoEquipamento'], order: { descricao: 'ASC' } })
            return res.json(equipamentos);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Equipamentos);
            const equipamento = await repository.findOne({ where: { id }, relations: ['imagens'] });
            if (!equipamento) {
                return res.status(404).json({ message: "Equipamento não encontrado" });
            }
            return res.json(equipamento);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { descricao, modelo, patrimonio, codigo, urlVideo, situacao, tipoEquipamento } = req.body;
            let imagens = [];
            let existeRegistro;
            console.log('Tipo Equipamento', tipoEquipamento);

            const repository = getRepository(Equipamentos);

            if (codigo) {
                existeRegistro = await repository.findOne({ where: { codigo } });
                if (existeRegistro) {
                    return res.status(400).json("Já existe um equipamento com este número de série.");
                }
            }

            if (patrimonio) {
                existeRegistro = await repository.findOne({ where: { patrimonio } });
                if (existeRegistro) {
                    return res.status(400).json("Já existe um equipamento com este patrimônio.");
                }
            }

            const equipamento = repository.create(req.body);
            const result: any = await repository.save(equipamento);

            if (req.file) {
                const directory = path.join(__dirname, '..', '..', 'uploads', 'equipamentos', `${result.id}`);
                if (!fs.existsSync(directory)) {
                    fs.mkdirSync(directory, { recursive: true });
                }


                const fileExtension = req.file.originalname.split('.').pop();
                const fileName = `${new Date().getTime().toString()}.${fileExtension}`;
                const filePath = path.join(directory, fileName);

                await fs.writeFileSync(filePath, req.file.buffer);

                result.urlImagem = path.relative(path.join(__dirname, '..', '..'), filePath);
                repository.save(result);
            }

            if (+tipoEquipamento != 1) {
                let diretorio_upload;
                switch (+tipoEquipamento) {
                    case 2:
                        diretorio_upload = 'itensveiculo';
                        break;
                    case 3:
                        diretorio_upload = 'implementos';
                        break;
                    default: diretorio_upload = '';
                        break;
                }
                if (req.files && Array.isArray(req.files)) {
                    const directory = path.join(__dirname, '..', '..', 'uploads', diretorio_upload, `${result.id}`);
                    if (!fs.existsSync(directory)) {
                        fs.mkdirSync(directory, { recursive: true });
                    }

                    for (const file of req.files) {
                        const fileExtension = file.originalname.split('.').pop(); // Obter a extensão do arquivo
                        const fileName = `${new Date().getTime().toString()}.${fileExtension}`; // Nome do arquivo com a extensão
                        const filePath = path.join(directory, fileName);

                        await fs.writeFileSync(filePath, file.buffer);

                        // Cria a entrada de imagem no banco de dados
                        const imagemRepository = getRepository(Imagens);
                        const imagem = imagemRepository.create({
                            path: path.relative(path.join(__dirname, '..', '..'), filePath),
                            equipamento_id: result.id
                        });
                        await imagemRepository.save(imagem);
                        imagens.push(imagem);
                    }
                }
            }

            return res.status(200).json(result);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o equipamento. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, modelo, patrimonio, codigo, urlVideo, situacao, tipoEquipamento, imagensRemovidas } = req.body;
            console.log('Tipo Equipamento', tipoEquipamento);

            const repository = getRepository(Equipamentos);

            let imagens = [];
            let existeRegistro;

            if (patrimonio) {
                existeRegistro = await repository.findOne({ where: { patrimonio } });
                if (existeRegistro && existeRegistro.id !== Number(id)) {
                    return res.status(400).json("Já existe um equipamento com este patrimônio.");
                }
            }

            if (codigo) {
                existeRegistro = await repository.findOne({ where: { codigo } });
                if (existeRegistro && existeRegistro.id !== Number(id)) {
                    return res.status(400).json("Já existe um equipamento com este número de série.");
                }
            }

            const equipamentoToUpdate = await repository.findOne({ where: { id }, relations: ['tipoEquipamento'] });
            if (!equipamentoToUpdate) {
                return res.status(404).json("Equipamento não encontrado");
            }
            if (equipamentoToUpdate.tipoEquipamento.id !== 1) {
                existeRegistro = await repository.findOne({ where: { descricao } });
                if (existeRegistro && existeRegistro.id !== Number(id)) {
                    return res.status(400).json("Já existe um equipamento com esta descrição.");
                }

            }

            repository.merge(equipamentoToUpdate, req.body);
            const result = await repository.save(equipamentoToUpdate);

            const imagemRepository = getRepository(Imagens);

            if (req.file) {
                const directory = path.join(__dirname, '..', '..', 'uploads', 'equipamentos', `${id}`);
                if (!fs.existsSync(directory)) {
                    fs.mkdirSync(directory, { recursive: true });
                }


                const fileExtension = req.file.originalname.split('.').pop();
                const fileName = `${new Date().getTime().toString()}.${fileExtension}`;
                const fileNameOriginal = `${new Date().getTime().toString()}.${fileExtension}`;
                const filePath = path.join(directory, fileName);
                const filePathOriginal = path.join(directory, fileNameOriginal);
                if (!fs.existsSync(filePathOriginal)) {
                    await fs.writeFileSync(filePath, req.file.buffer);

                    result.urlImagem = path.relative(path.join(__dirname, '..', '..'), filePath);
                    repository.save(result);

                } else {
                    await fs.writeFileSync(filePathOriginal, req.file.buffer);
                }



            }

            if (+tipoEquipamento != 1) {
                let diretorio_upload;
                switch (+tipoEquipamento) {
                    case 2:
                        diretorio_upload = 'itensveiculo';
                        break;
                    case 3:
                        diretorio_upload = 'implementos';
                        break;
                    default: diretorio_upload = '';
                        break;
                }

                if (imagensRemovidas) {
                    if (Array.isArray(imagensRemovidas)) {
                        for (const imagem of imagensRemovidas) {
                            const startIndex = imagem.indexOf('uploads');
                            const filePath = imagem.substring(startIndex);
                            console.log('URL', filePath);
                            await imagemRepository.softDelete({ path: filePath });
                        }
                    } else {
                        const startIndex = imagensRemovidas.indexOf('uploads');
                        const filePath = imagensRemovidas.substring(startIndex);
                        console.log('URL', filePath);
                        await imagemRepository.softDelete({ path: filePath });
                    }
                }


                if (req.files && Array.isArray(req.files)) {
                    const directory = path.join(__dirname, '..', '..', 'uploads', diretorio_upload, `${id}`);
                    if (!fs.existsSync(directory)) {
                        fs.mkdirSync(directory, { recursive: true });
                    }

                    for (const file of req.files) {
                        const fileExtension = file.originalname.split('.').pop();
                        const fileName = `${new Date().getTime().toString()}.${fileExtension}`;
                        const fileNameOriginal = `${new Date().getTime().toString()}.${fileExtension}`;
                        const filePath = path.join(directory, fileName);
                        const filePathOriginal = path.join(directory, fileNameOriginal);
                        if (!fs.existsSync(filePathOriginal)) {
                            await fs.writeFileSync(filePath, file.buffer);


                            const imagemRepository = getRepository(Imagens);
                            const imagem = imagemRepository.create({
                                path: path.relative(path.join(__dirname, '..', '..'), filePath),
                                equipamento_id: +id
                            });
                            await imagemRepository.save(imagem);
                            imagens.push(imagem);
                        } else {
                            console.log('Arquivo existente');
                        }


                    }
                }
            }

            return res.json(result);
        } catch (error) {
            console.error('Update equipamento', error);
            return res.status(500).json("Erro ao salvar o equipamento. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Equipamentos);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Equipamento não encontrado');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir a equipamento. Tente novamente.");
        }
    }

    static async listEquipamentosUnidade(req: Request, res: Response): Promise<Response> {
        const { tipo_equipamento_id, viatura_id } = req.params;
        try {
            const repository = getRepository(UnidadesMateriais);
            const viaturaRepository = getRepository(Viaturas);
            const equipamentosRepository = getRepository(Equipamentos);

            // Buscando a viatura e a unidade correspondente
            const viatura = await viaturaRepository.findOne({ where: { id: viatura_id }, relations: ['unidade'] });
            if (!viatura) {
                return res.status(404).json({ error: 'Viatura não encontrada' });
            }

            // Buscando todos os UnidadesMateriais correspondentes
            const unidadesMateriais = await repository.find({
                where: {
                    equipamento: { tipoEquipamento: tipo_equipamento_id },
                    unidade: viatura.unidade.id
                },
                relations: ['equipamento']
            });

            // Extraindo os IDs dos equipamentos
            //const equipamentoIds = unidadesMateriais.map(um => um.equipamento.id);

            // Buscando todos os equipamentos usando os IDs
            //const equipamentos = await equipamentosRepository.findByIds(equipamentoIds);

            return res.json(unidadesMateriais);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }
}

export default EquipamentoController;
