import { Request, Response } from 'express';
import { getRepository, In } from 'typeorm';
import { Categorias } from '../entities/Categorias';
import { InventariosMateriais } from '../entities/InventariosMateriais';
import { where } from 'sequelize';
import { Inventarios } from '../entities/Inventarios';

class InventarioMaterialController {
    static async list(req: Request, res: Response): Promise<Response> {
        try {
            const repository = getRepository(InventariosMateriais);
            const inventariosMateriais = await repository.find({ order: { createdAt: 'ASC' } })
            return res.json(inventariosMateriais);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(InventariosMateriais);
            const inventarioMaterial = await repository.findOne({ where: { id }, relations: ["inventario", "equipamento", "categoria"] });
            if (!inventarioMaterial) {
                return res.status(404).json({ message: "Inventario Material não encontrado" });
            }
            return res.json(inventarioMaterial);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { material } = req.body;

            console.log("Add material", req.body);

            const repository = getRepository(InventariosMateriais);

            const inventarioMaterial = new InventariosMateriais();
            inventarioMaterial.categoria = material.categoria.id;
            inventarioMaterial.equipamento = material.equipamento.id;
            inventarioMaterial.quantidade = material.quantidade;
            inventarioMaterial.inventario = material.inventario.id;
            inventarioMaterial.unidadeMaterial = material.unidadeMaterial.id;

            const result = await repository.save(inventarioMaterial);

            const ivnMat = await repository.findOne({ where: { id: result.id }, relations: ["inventario", "equipamento", "categoria", "unidadeMaterial"] });

            console.log('Material adicionado', inventarioMaterial);


            return res.status(200).json(ivnMat);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o material. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { material } = req.body;

            console.log('Params', req.params);

            console.log("Update material", req.body);
            const repository = getRepository(InventariosMateriais);

            const ivnMat = await repository.findOne({ where: { id: id } });
            console.log('Inventario Material', ivnMat);

            ivnMat.categoria = material.categoria.id;
            ivnMat.equipamento = material.equipamento.id;
            ivnMat.quantidade = material.quantidade;
            ivnMat.unidadeMaterial = material.unidadeMaterial.id;

            const inventarioMaterial = await repository.update(id, ivnMat);

            return res.status(200).json('Item atualizado.');
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o material. Tente novamente.');
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(InventariosMateriais);

            const ivnMat = await repository.findOne({ where: { id: id } });
            if (ivnMat) {
                await repository.softRemove(ivnMat);
                return res.status(200).json('Item removido.');
            } else {
                return res.status(400).json('O item não foi encontrado');
            }
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o material. Tente novamente.');
        }
    }

    static async listarPorIds(req: Request, res: Response): Promise<Response> {
        const { ids, unidade } = req.body;
        console.log('Ids', ids);
        try {
            const repository = getRepository(InventariosMateriais);
            const inventariosMateriais = await repository.find({ where: { unidadeMaterial: { id: In(ids) }, inventario: { viatura: { unidade: unidade } } }, relations: ["unidadeMaterial", "equipamento", "categoria", "inventario", "inventario.viatura"], order: { createdAt: 'ASC' } })
            return res.json(inventariosMateriais);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async salvarMovimentacao(req: Request, res: Response): Promise<Response> {
        try {
            const { unidadeId,
                materiaisSelecionados,
                itensRemovidos } = req.body;

            console.log("Movimentação", req.body);
            const repository = getRepository(InventariosMateriais);
            const repositoryInventarios = getRepository(Inventarios);


            for (const r of itensRemovidos) {
                console.log('R:', r)
                const ivnMat = await repository.findOne({ where: { id: r } });
                if (ivnMat) {
                    await repository.softRemove(ivnMat);
                }
            }

            for (const material of materiaisSelecionados) {
                if (material.id == null) {

                    const inventario = await repositoryInventarios.findOne({ where: { viatura: material.viatura_id, tipoEquipamento: material.equipamento.tipoEquipamento.id } });
                    console.log('Iventario encontrado:', inventario);
                    let result;
                    if (inventario == null) {
                        console.log('Iventario não encontrado:');
                        const inventario = new Inventarios();
                        inventario.descricao = `Checklist ${material.equipamento.tipoEquipamento.descricao} ${material.viatura}`;
                        inventario.viatura = material.viatura_id;
                        inventario.tipoEquipamento = material.equipamento.tipoEquipamento;
                        inventario.situacao = 'Ativo';

                        result = await repositoryInventarios.save(inventario);
                        console.log('Novo Iventario:', result);
                    }

                    const inventarioMaterial = new InventariosMateriais();
                    inventarioMaterial.categoria = material.categoria_id;
                    inventarioMaterial.equipamento = material.equipamento.id;
                    inventarioMaterial.quantidade = material.quantidade;
                    inventarioMaterial.inventario = inventario ?? result;
                    inventarioMaterial.unidadeMaterial = material.unidade_material_id;
                    await repository.save(inventarioMaterial);
                }
            }

            return res.status(200).json('Movimentação realizada.');
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao movimentar material. Tente novamente.');
        }
    }
}

export default InventarioMaterialController;
