import { Request, Response } from 'express';
import { getManager, getRepository, In } from 'typeorm';
import { ItensCheckList } from '../entities/ItensCheckList';

class RelatorioController {
    static async graficoEquipamentoEmOperacao(req: Request, res: Response): Promise<Response> {
        const { unidades } = req.body;
        try {
            const entityManager = getManager();
            const rawQuery = `
                WITH latest_checklist AS (
                    SELECT 
                        c.viatura_id,
                        MAX(c.data) AS data_recente
                    FROM 
                        checklists c
                    WHERE
                        c.deleted_at IS NULL
                    GROUP BY 
                        c.viatura_id
                ),
                recent_checklist AS (
                    SELECT 
                        c.id AS checklist_id,
                        c.viatura_id
                    FROM 
                        checklists c
                    JOIN 
                        latest_checklist lc ON c.viatura_id = lc.viatura_id AND c.data = lc.data_recente
                    WHERE
                        c.deleted_at IS NULL
                ),
                unidades_materiais_summed AS (
                    SELECT 
                        um.unidade_id,
                        um.equipamento_id,
                        um.numero_serie,
                        SUM(um.quantidade) AS quantidade_total
                    FROM 
                        unidades_materiais um
                    WHERE
                        um.deleted_at IS NULL
                        AND um.unidade_id IN (${unidades}) 
                    GROUP BY 
                        um.unidade_id, um.equipamento_id, um.numero_serie
                ),
                itens_checklist_summed AS (
                    SELECT 
                        v.unidade_id,
                        ic.equipamento_id,
                        SUM(ic.quantidade) AS quantidade_operacao
                    FROM 
                        itens_checklist ic
                    JOIN 
                        recent_checklist rc ON ic.checklist_id = rc.checklist_id
                    JOIN 
                        viaturas v ON rc.viatura_id = v.id
                    WHERE
                        ic.deleted_at IS NULL
                        AND v.deleted_at IS NULL
                    GROUP BY 
                        v.unidade_id, ic.equipamento_id
                )
                SELECT 
                    u.id AS unidade_id,
                    u.nome AS unidade_nome,
                    e.id AS equipamento_id,
                    e.descricao AS equipamento_nome,
                    COALESCE(um.quantidade_total, 0) AS total_disponivel,
                    COALESCE(ic.quantidade_operacao, 0) AS total_em_operacao,
                    CASE 
                        WHEN COALESCE(um.quantidade_total, 0) = 0 THEN 0
                        ELSE (COALESCE(ic.quantidade_operacao, 0) * 100.0 / COALESCE(um.quantidade_total, 0))
                    END AS percentual_operacao
                FROM 
                    unidades_materiais_summed um
                JOIN 
                    unidades u ON um.unidade_id = u.id AND u.deleted_at IS NULL
                LEFT JOIN 
                    itens_checklist_summed ic ON um.equipamento_id = ic.equipamento_id AND um.unidade_id = ic.unidade_id
                JOIN 
                    equipamentos e ON um.equipamento_id = e.id AND e.deleted_at IS NULL
                WHERE 
                    um.unidade_id IN (${unidades}) 
                ORDER BY 
                    u.id, e.id;
            `;

            const result = await entityManager.query(rawQuery);

            const aggregatedResults = result.reduce((acc, curr) => {
                const equipamentoId = curr.equipamento_id;
                if (!acc[equipamentoId]) {
                    acc[equipamentoId] = {
                        equipamento_id: equipamentoId,
                        equipamento_nome: curr.equipamento_nome,
                        total_disponivel: 0,
                        total_em_operacao: 0
                    };
                }
                acc[equipamentoId].total_disponivel += parseFloat(curr.total_disponivel);
                acc[equipamentoId].total_em_operacao += parseFloat(curr.total_em_operacao);
                return acc;
            }, {});

            const finalResults = Object.values(aggregatedResults);

            return res.json(finalResults);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async checklistsIncompletosPorUnidades(req: Request, res: Response): Promise<Response> {
        const { unidades, data } = req.body;
        try {
            const entityManager = getManager();
            const rawQuery = `WITH total_checklists AS (
                                SELECT 
                                    u.id AS unidade_id,
                                    COUNT(DISTINCT v.id) * 3 AS total_checklists
                                FROM unidades u
                                JOIN viaturas v ON u.id = v.unidade_id
                                GROUP BY u.id
                            ),
                            realizados_checklists AS (
                                SELECT 
                                    u.id AS unidade_id,
                                    a.nome as CBA,
                                    SUM(CASE WHEN c.tipo_equipamento_id = 1 THEN 1 ELSE 0 END) AS realizados_equipamento,
                                    SUM(CASE WHEN c.tipo_equipamento_id = 2 THEN 1 ELSE 0 END) AS realizados_itens_viatura,
                                    SUM(CASE WHEN c.tipo_equipamento_id = 3 THEN 1 ELSE 0 END) AS realizados_implemento
                                FROM unidades u
                                JOIN viaturas v ON u.id = v.unidade_id
                                LEFT JOIN area a ON u.area_id = a.id
                                LEFT JOIN checklists c ON v.id = c.viatura_id AND c.data = '${data}'
                                GROUP BY u.id, a.nome
                            )
                            SELECT 
                                u.nome AS unidade,
                                a.nome as CBA,
                                COALESCE(total.total_checklists, 0) AS total_checklists,
                                COALESCE(realizados.realizados_equipamento, 0) AS realizados_equipamento,
                                COALESCE(realizados.realizados_itens_viatura, 0) AS realizados_itens_viatura,
                                COALESCE(realizados.realizados_implemento, 0) AS realizados_implemento,
                                (COALESCE(total.total_checklists, 0) / 3 - COALESCE(realizados.realizados_equipamento, 0)) AS nao_realizados_equipamento,
                                (COALESCE(total.total_checklists, 0) / 3 - COALESCE(realizados.realizados_itens_viatura, 0)) AS nao_realizados_itens_viatura,
                                (COALESCE(total.total_checklists, 0) / 3 - COALESCE(realizados.realizados_implemento, 0)) AS nao_realizados_implemento,
                                CASE 
                                    WHEN COALESCE(total.total_checklists, 0) = 0 THEN 0
                                    ELSE ROUND((COALESCE(total.total_checklists, 0) / 3 - COALESCE(realizados.realizados_equipamento, 0))::NUMERIC / (COALESCE(total.total_checklists, 1) / 3) * 100, 2) 
                                END AS percentual_nao_realizados_equipamento,
                                CASE 
                                    WHEN COALESCE(total.total_checklists, 0) = 0 THEN 0
                                    ELSE ROUND((COALESCE(total.total_checklists, 0) / 3 - COALESCE(realizados.realizados_itens_viatura, 0))::NUMERIC / (COALESCE(total.total_checklists, 1) / 3) * 100, 2) 
                                END AS percentual_nao_realizados_itens_viatura,
                                CASE 
                                    WHEN COALESCE(total.total_checklists, 0) = 0 THEN 0
                                    ELSE ROUND((COALESCE(total.total_checklists, 0) / 3 - COALESCE(realizados.realizados_implemento, 0))::NUMERIC / (COALESCE(total.total_checklists, 1) / 3) * 100, 2) 
                                END AS percentual_nao_realizados_implemento
                            FROM unidades u
                            LEFT JOIN area a ON u.area_id = a.id
                            LEFT JOIN total_checklists total ON u.id = total.unidade_id
                            LEFT JOIN realizados_checklists realizados ON u.id = realizados.unidade_id
                            WHERE u.id IN (${unidades}) 
                            ORDER BY u.nome, a.nome;
                                        `;

            const result = await entityManager.query(rawQuery);

            return res.json(result);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async equipamentosInoperantes(req: Request, res: Response): Promise<Response> {
        const { unidades, cbas } = req.body;
        console.log('Body', req.body);
        try {
            const repository = getRepository(ItensCheckList);
            const result = await repository.createQueryBuilder('ic')
                .select([
                    'u.nome AS unidade',
                    'a.nome AS CBA',
                    'e.id AS equipamento_id',
                    'e.descricao AS equipamento',
                    'e.modelo AS modelo',
                    'e.tipoEquipamento.id AS tipo_equipamento_id',
                    'te.descricao as tipoEquipamento',
                    'um.numeroSerie AS numeroSerie',
                    'ic.descricaoPendencia AS descricaoPendencia',
                    'ic.quantidade AS quantidadeReal',
                    'im.quantidade AS quantidadePrevista',
                    'im.id as inventario_material_id',
                    'cl.data as dataUltimoCheckList'
                ])
                .innerJoin('ic.checklist', 'cl')
                .innerJoin('ic.inventarioMaterial', 'im')
                .innerJoin('im.unidadeMaterial', 'um')
                .innerJoin('um.unidade', 'u')
                .leftJoin('u.area', 'a')
                .innerJoin('ic.equipamento', 'e')
                .innerJoin('e.tipoEquipamento', 'te')
                .where((qb) => {
                    const subQuery = qb.subQuery()
                        .select([
                            'u2.nome',
                            'e2.descricao',
                            'MAX(cl2.data)'
                        ])
                        .from(ItensCheckList, 'ic2')
                        .innerJoin('ic2.checklist', 'cl2')
                        .innerJoin('ic2.inventarioMaterial', 'im2')
                        .innerJoin('im2.unidadeMaterial', 'um2')
                        .innerJoin('um2.unidade', 'u2')
                        .innerJoin('ic2.equipamento', 'e2')
                        .where('u2.id IN (:...unidades)', { unidades })
                        .groupBy('u2.nome, e2.descricao')
                        .getQuery();
                    return `("u"."nome", "e"."descricao", "cl"."data") IN ${subQuery}`;
                })
                .andWhere('ic.descricaoPendencia IS NOT NULL OR (ic.quantidade IS NOT NULL AND ic.quantidade < im.quantidade)')
                .andWhere('u.id IN (:...unidades)', { unidades })
                //.andWhere('a.id IN (:...cbas)', { cbas })
                .orderBy('u.nome')
                .addOrderBy('e.descricao')
                .getRawMany();

            return res.json(result);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }
}

export default RelatorioController;
