import { Request, Response } from 'express';
import { getRepository } from 'typeorm';
import { TiposViatura } from '../entities/TiposViatura';
import { Categorias } from '../entities/Categorias';

class TiposViaturaController {
    static async list(req: Request, res: Response): Promise<Response> {
        try {
            const repository = getRepository(TiposViatura);
            const tiposViatura = await repository.find({ order: { nome: 'ASC' }, relations: ['categorias'] });
            return res.json(tiposViatura);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(TiposViatura);
            const tipoViatura = await repository.findOne(id, { relations: ['categorias'] });
            if (!tipoViatura) {
                return res.status(404).json({ message: "Tipo de Viatura não encontrado" });
            }
            return res.json(tipoViatura);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { nome, categoriasIds } = req.body;
            const repository = getRepository(TiposViatura);
            const categoriasRepository = getRepository(Categorias);

            const categorias = await categoriasRepository.findByIds(categoriasIds);
            const tipoViatura = repository.create({
                nome,
                categorias
            });

            const result = await repository.save(tipoViatura);
            return res.status(200).json(result);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o Tipo de Viatura. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { nome, categoriasIds } = req.body;
            const repository = getRepository(TiposViatura);
            const categoriasRepository = getRepository(Categorias);

            const tipoViaturaToUpdate = await repository.findOne(id, { relations: ['categorias'] });
            if (!tipoViaturaToUpdate) {
                return res.status(404).json("Tipo de Viatura não encontrado");
            }

            const categorias = await categoriasRepository.findByIds(categoriasIds);

            repository.merge(tipoViaturaToUpdate, {
                nome,
                categorias
            });

            const result = await repository.save(tipoViaturaToUpdate);
            return res.json(result);
        } catch (error) {
            return res.status(500).json("Erro ao salvar o Tipo de Viatura. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(TiposViatura);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Tipo de Viatura não encontrado');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir o Tipo de Viatura. Tente novamente.");
        }
    }
}

export default TiposViaturaController;
