import { Request, Response } from 'express';
import { getRepository, In } from 'typeorm';
import { Unidades } from '../entities/Unidades';
import { UnidadesMateriais } from '../entities/UnidadesMateriais';
import { User } from '../entities/User';

class UnidadeController {
    static async list(req: Request, res: Response): Promise<Response> {
        const { userId } = req.body;
        try {
            const repository = getRepository(Unidades);
            const repositoryUsuario = getRepository(User);

            const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });
            if (!usuario) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }

            const unidadesIds = usuario.unidades.map(unidade => unidade.id);

            const unidades = await repository.find({
                where: { id: In(unidadesIds) },
                relations: ["area", "subarea", "materiais", "materiais.equipamento", "materiais.equipamento.tipoEquipamento"], order: {
                    ordem: "ASC",

                }
            });
            return res.json(unidades);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        const { userId } = req.body;
        try {
            const repositoryUsuario = getRepository(User);

            const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });
            if (!usuario) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }

            const unidadesIds = usuario.unidades.map(unidade => unidade.id);

            const { id } = req.params;
            console.log('Id unidade', id);

            if (!unidadesIds.includes(+id)) {
                return res.status(403).json({ message: "Usuário não tem permissão de acesso a esta unidade" });
            }

            const repository = getRepository(Unidades);
            const unidade = await repository.findOne(id, { relations: ["area", "subarea", "materiais", "materiais.equipamento", "materiais.equipamento.tipoEquipamento"] });
            console.log('Unidade', unidade);
            if (!unidade) {
                return res.status(404).json({ message: "Unidade não encontrada" });
            }
            return res.json(unidade);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { nome, ordem } = req.body;
            const repository = getRepository(Unidades);

            let existeRegistro = await repository.findOne({ where: { nome } });
            if (existeRegistro) {
                return res.status(400).json("Já existe uma unidade com este nome.");
            }
            existeRegistro = await repository.findOne({ where: { ordem } });
            if (existeRegistro) {
                return res.status(400).json("Já existe uma unidade com esta ordem.");
            }


            const unidade = repository.create(req.body);
            const result = await repository.save(unidade);
            return res.status(200).json(result);
        } catch (error) {
            return res.status(500).json('Erro ao salvar a unidade. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { nome, userId } = req.body;

            const repository = getRepository(Unidades);

            const repositoryUsuario = getRepository(User);

            const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });
            if (!usuario) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }

            const unidadesIds = usuario.unidades.map(unidade => unidade.id);

            if (!unidadesIds.includes(+id)) {
                return res.status(403).json({ message: "Usuário não tem permissão de acesso a esta unidade" });
            }

            const existeRegistro = await repository.findOne({ where: { nome } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe uma unidade com este nome.");
            }

            const unidadeToUpdate = await repository.findOne(id);
            if (!unidadeToUpdate) {
                return res.status(404).json("Unidade não encontrada");
            }
            delete req.body.userId;
            if (req.body.subarea == '' || req.body.subarea == null || req.body.subarea == undefined) {
                delete req.body.subarea;
            }

            repository.merge(unidadeToUpdate, req.body);
            const result = await repository.save(unidadeToUpdate);
            return res.json(result);
        } catch (error) {
            return res.status(500).json("Erro ao salvar a unidade. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Unidades);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Unidade não encontrada');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir a unidade. Tente novamente.");
        }
    }

    static async salvarInventario(req: Request, res: Response): Promise<Response> {
        try {
            const { equipamentos, unidade, itensRemovidos } = req.body;
            console.log('Body', req.body);

            const repositoryMaterial = getRepository(UnidadesMateriais);

            for (const e of equipamentos) {
                if (!e?.equipamento) {
                    const inventarioMaterial = new UnidadesMateriais();
                    inventarioMaterial.unidade = unidade.id;
                    inventarioMaterial.quantidade = e.quantidade;
                    inventarioMaterial.equipamento = e.id;
                    inventarioMaterial.patrimonio = e.patrimonio;
                    inventarioMaterial.numeroSerie = e.codigo;

                    await repositoryMaterial.save(inventarioMaterial);
                }
            }

            for (const i of itensRemovidos) {
                await repositoryMaterial.softDelete(i.id);
            }

            return res.status(200).json("Inventario criado.");
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar o inventário. Tente novamente.');
        }
    }

    static async listarPorCBA(req: Request, res: Response): Promise<Response> {
        const { userId } = req.body;
        try {
            const { cba_id } = req.params;
            const repository = getRepository(Unidades);
            const repositoryUsuario = getRepository(User);

            const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });
            if (!usuario) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }

            const unidadesIds = usuario.unidades.map(unidade => unidade.id);

            const unidades = await repository.find({
                where: { id: In(unidadesIds), area: cba_id },
                relations: ["area", "subarea"], order: {
                    ordem: "ASC",

                }
            });
            return res.json(unidades);
        } catch (error) {
            return res.status(500).json({ error: error.message });
        }
    }
}

export default UnidadeController;
