// src/controllers/authController.ts
import { Request, Response } from 'express';
import { UserService } from '../services/UserService';
import { PermissionRepository, RoleRepository, UserRepository } from '../repositories';
import { hash } from "bcrypt";
import { getRepository } from 'typeorm';
import { request } from 'http';
import { Unidades } from '../entities/Unidades';
import { User } from '../entities/User';
import bcrypt from 'bcrypt';
import { comparePassword, hashPassword } from '../services/authService';

const validatePassword = (password: string): boolean => {
    const regex = /^(?=.*[A-Z])(?=.*[!@#$%^&*])(?=.{8,})(?!.*\s)/;
    return regex.test(password);
};

export class UserController {


    async rolesUser(req: Request, res: Response) {
        try {
            const { userId } = req.body;

            const userService = new UserService();

            const result = await userService.execute({
                userId,
            });

            if (result instanceof Error) {
                return res.status(400).json(result.message);
            }

            return res.json(result);
        } catch (error) {
            console.error(error);
            res.status(500).json({ message: 'Erro no processo de buscar roles' });
        }
    };

    async users(req: Request, res: Response) {
        const { userId } = req.body;
        try {
            const repository = UserRepository();
            const unidadesRepository = getRepository(Unidades);

            const usuarioLogado = await repository.findOne({ where: { id: userId }, relations: ['unidades', 'roles'] });
            if (!usuarioLogado) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }

            const allUnidades = await unidadesRepository.find();
            const allUnidadesIds = allUnidades.map(unidade => unidade.id);
            const unidadesIds = usuarioLogado.unidades.map(unidade => unidade.id);

            let isAdmin = usuarioLogado.roles.some(role => role.name === 'Administrador');
            let usuarios;

            if (isAdmin) {
                // Usuário administrador: listar todos os usuários
                usuarios = await repository.find({ where: { deletedAt: null }, relations: ["roles", "permissions", "unidades"] });
            } else {
                // Usuário não administrador: listar usuários apenas das unidades permitidas e sem perfil administrador
                usuarios = await repository.createQueryBuilder('user')
                    .leftJoinAndSelect('user.roles', 'roles')
                    .leftJoinAndSelect('user.permissions', 'permissions')
                    .leftJoinAndSelect('user.unidades', 'unidades')
                    .where('user.deletedAt IS NULL')
                    .andWhere('unidades.id IN (:...unidadesIds)', { unidadesIds })
                    .andWhere('roles.name != :adminRole', { adminRole: 'Administrador' })
                    .andWhere('user.id != :userId', { userId: userId })
                    .getMany();
            }

            res.json(usuarios);
        } catch (error) {
            console.error(error);
            res.status(500).send('Erro ao buscar usuários.');
        }
    }

    async showUser(req: Request, res: Response) {
        const { id } = req.params;
        try {

            const repository = UserRepository()
            const usuario = await repository.findOne(id, { relations: ["roles", "permissions", "unidades"] });

            res.json(usuario);
        } catch (error) {
            console.error(error);
            res.status(500).send(`Erro ao buscar usuario ${id}.`);
        }
    };

    async createUser(req: Request, res: Response) {
        const { nome, rg, celular, email, perfil, senha, unidades } = req.body;
        try {

            console.log('Body create', req.body);
            const repo = UserRepository();
            const repoUnidades = getRepository(Unidades);

            let existeUsuario = await repo.findOne({ where: { rg } });
            if (existeUsuario) {
                return res.status(400).send("Já existe um usuário com este RG.");
            }
            existeUsuario = await repo.findOne({ where: { email } });
            if (existeUsuario) {
                return res.status(400).send("Já existe um usuário com este e-mail.");
            }

            const usuario = new User();

            if (perfil) {
                const perfilExistente = await RoleRepository().find({ where: { id: perfil } });
                usuario.roles = perfilExistente;
            }
            usuario.fullName = nome;
            usuario.username = nome.trim();
            usuario.rg = rg;
            usuario.email = email;
            usuario.celular = celular;
            usuario.status = 'Ativo';
            usuario.primeiroAcesso = true;

            const passwordHash = await hash(senha, 8);
            usuario.password = passwordHash;

            if (unidades != null && unidades != undefined) {
                const existeUnidades = await repoUnidades.findByIds(unidades);
                usuario.unidades = existeUnidades;
            }

            const result = await repo.save(usuario);


            return res.status(200).json('Usuário criado com sucesso.');

        } catch (error) {
            console.error("Erro ao criar usuário:", error);
            res.status(500).send("Erro ao criar usuário.");
        }
    }

    async editUser(req: Request, res: Response) {
        const { id } = req.params;
        const { nome, rg, celular, email, perfil, senha, unidades, situacao } = req.body;
        try {

            console.log('Body create', req.body);
            const repo = UserRepository();
            const repoUnidades = getRepository(Unidades);

            let existeUsuario = await repo.findOne({ where: { rg } });
            if (existeUsuario && existeUsuario.id !== id) {
                return res.status(400).send("Já existe um usuário com este RG.");
            }
            existeUsuario = await repo.findOne({ where: { email } });
            if (existeUsuario && existeUsuario.id !== id) {
                return res.status(400).send("Já existe um usuário com este e-mail.");
            }

            const usuario = await repo.findOne(id);

            if (perfil) {
                const perfilExistente = await RoleRepository().find({ where: { id: perfil } });
                usuario.roles = perfilExistente;
            }
            usuario.fullName = nome;
            usuario.username = nome.trim();
            usuario.rg = rg;
            usuario.email = email;
            usuario.celular = celular;
            usuario.status = situacao;

            if (senha) {
                const passwordHash = await hash(senha, 8);
                usuario.password = passwordHash;
            }

            if (unidades != null && unidades != undefined) {
                const existeUnidades = await repoUnidades.findByIds(unidades);
                usuario.unidades = existeUnidades;
            }

            const result = await repo.save(usuario);


            return res.status(200).json('Usuário atualizado com sucesso.');

        } catch (error) {
            console.error("Erro ao atualizar usuário:", error);
            res.status(500).send("Erro ao atualizar usuário.");
        }
    }

    async user(req: Request, res: Response) {

        const { userId } = req.body;

        try {

            const repository = UserRepository()
            const usuario = await repository.findOne(userId, { relations: ["roles", "permissions", "unidades"] });
            delete usuario.password;
            res.json(usuario);
        } catch (error) {
            console.error(error);
            res.status(500).send(`Erro ao buscar usuario ${userId}.`);
        }
    };

    async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = UserRepository();
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Usuário não encontrado');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir o usuário. Tente novamente.");
        }
    }

    async changePassword(req: Request, res: Response) {
        const { userId, senhaAtual, senhaNova } = req.body;
        console.log('Body user', req.body);
        try {
            if (!validatePassword(senhaNova)) {
                return res.status(400).json("A nova senha deve ter no mínimo 8 caracteres, uma letra maiúscula, um caractere especial e não pode conter espaços.");
            }

            const repository = UserRepository()
            const usuario = await repository.findOne({ where: { id: userId } });

            if (!usuario || !(await comparePassword(senhaAtual, usuario.password))) {
                return res.status(400).json("Não foi possível trocar a senha, verifique as senhas enviadas.");
            }

            const repo = UserRepository();
            const senha = await hash(senhaNova, 8);
            usuario.password = senha;
            usuario.primeiroAcesso = false;
            const result = await repo.save(usuario);
            res.json('Senha alterada com sucesso.');
        } catch (error) {
            console.error(error);
            res.status(500).send(`Erro ao alterar a senha do usuario ${userId}.`);
        }
    };
}



