import { Request, Response } from 'express';
import { In, getRepository } from 'typeorm';
import { Viaturas } from '../entities/Viaturas';
import fs from 'fs';
import path from 'path';
import { promisify } from 'util';
import { Imagens } from '../entities/Imagens';
import { where } from 'sequelize';
import { User } from '../entities/User';

const renameAsync = promisify(fs.rename);

class ViaturaController {
    static async list(req: Request, res: Response): Promise<Response> {
        const { userId } = req.body;
        try {
            const repositoryUsuario = getRepository(User);
            const repository = getRepository(Viaturas);

            const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });
            if (!usuario) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }

            const unidadesIds = usuario.unidades.map(unidade => unidade.id);

            const viaturas = await repository.find({
                where: {
                    unidade: {
                        id: In(unidadesIds)
                    }
                },
                relations: ['unidade', 'imagens'],
                order: { prefixo: 'ASC', numero: 'ASC' }
            });

            return res.json(viaturas);
        } catch (error) {
            return res.status(500).json(error.message);
        }
    }

    static async show(req: Request, res: Response): Promise<Response> {
        const { userId } = req.body;
        try {
            const repositoryUsuario = getRepository(User);
            const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });
            if (!usuario) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }

            const unidadesIds = usuario.unidades.map(unidade => unidade.id);
            const { id } = req.params;
            const repository = getRepository(Viaturas);
            const viatura = await repository.findOne({
                where: {
                    id, unidade: {
                        id: In(unidadesIds)
                    }
                }, relations: ["unidade", "imagens", "inventarios", "inventarios.tipoEquipamento", "inventarios.inventarioMateriais", "inventarios.inventarioMateriais.categoria", "inventarios.inventarioMateriais.equipamento", "inventarios.inventarioMateriais.unidadeMaterial"]
            });
            if (!viatura) {
                return res.status(400).json("Viatura não encontrada");
            }
            console.log('Viatura', viatura);
            return res.json(viatura);
        } catch (error) {
            console.error(error);
            return res.status(500).json(error.message);
        }
    }

    static async create(req: Request, res: Response): Promise<Response> {
        try {
            const { descricao, chassi, marca, modelo, prefixo, numero, unidade, urlVideo, situacao } = req.body;
            let imagens = [];

            console.log("Body create viatura", req.body);
            console.log('Files', req.files);

            const repository = getRepository(Viaturas);

            const existeRegistro = await repository.findOne({ where: { prefixo, numero } });
            if (existeRegistro) {
                return res.status(400).json("Já existe uma viatura com este número e prefixo.");
            }

            const viatura = repository.create(req.body);
            const result: any = await repository.save(viatura);

            if (req.files && Array.isArray(req.files)) {
                const directory = path.join(__dirname, '..', '..', 'uploads', 'viaturas', `${prefixo}-${numero}`);
                if (!fs.existsSync(directory)) {
                    fs.mkdirSync(directory, { recursive: true });
                }

                for (const file of req.files) {
                    const fileExtension = file.originalname.split('.').pop(); // Obter a extensão do arquivo
                    const fileName = `${new Date().getTime().toString()}.${fileExtension}`; // Nome do arquivo com a extensão
                    const filePath = path.join(directory, fileName);

                    await fs.writeFileSync(filePath, file.buffer);

                    // Cria a entrada de imagem no banco de dados
                    const imagemRepository = getRepository(Imagens);
                    const imagem = imagemRepository.create({
                        path: path.relative(path.join(__dirname, '..', '..'), filePath),
                        viatura_id: result.id
                    });
                    await imagemRepository.save(imagem);
                    imagens.push(imagem);
                }
            }

            return res.status(200).json(result);
        } catch (error) {
            console.error(error);
            return res.status(500).json('Erro ao salvar a viatura. Tente novamente.');
        }
    }

    static async update(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const { descricao, chassi, marca, modelo, prefixo, numero, unidade, urlVideo, situacao, imagensRemovidas } = req.body;
            console.log('Body', req.body);
            console.log('Imagens removidas', imagensRemovidas);
            console.log('Imagens inserir', req.files);
            const repository = getRepository(Viaturas);

            let imagens = [];

            const existeRegistro = await repository.findOne({ where: { prefixo, numero } });
            if (existeRegistro && existeRegistro.id !== Number(id)) {
                return res.status(400).json("Já existe uma viatura com este prefixo e número.");
            }

            const viaturaToUpdate = await repository.findOne(id);
            if (!viaturaToUpdate) {
                return res.status(404).json("Viatura não encontrada");
            }

            repository.merge(viaturaToUpdate, req.body);
            const result = await repository.save(viaturaToUpdate);

            const imagemRepository = getRepository(Imagens);
            if (imagensRemovidas) {
                if (Array.isArray(imagensRemovidas)) {
                    for (const imagem of imagensRemovidas) {
                        const startIndex = imagem.indexOf('uploads');
                        const filePath = imagem.substring(startIndex);
                        console.log('URL', filePath);
                        await imagemRepository.softDelete({ path: filePath });
                    }
                } else {
                    const startIndex = imagensRemovidas.indexOf('uploads');
                    const filePath = imagensRemovidas.substring(startIndex);
                    console.log('URL', filePath);
                    await imagemRepository.softDelete({ path: filePath });
                }
            }


            if (req.files && Array.isArray(req.files)) {
                const directory = path.join(__dirname, '..', '..', 'uploads', 'viaturas', `${prefixo}-${numero}`);
                if (!fs.existsSync(directory)) {
                    fs.mkdirSync(directory, { recursive: true });
                }

                for (const file of req.files) {
                    const fileExtension = file.originalname.split('.').pop();
                    const fileName = `${new Date().getTime().toString()}.${fileExtension}`;
                    const fileNameOriginal = `${new Date().getTime().toString()}.${fileExtension}`;
                    const filePath = path.join(directory, fileName);
                    const filePathOriginal = path.join(directory, fileNameOriginal);
                    if (!fs.existsSync(filePathOriginal)) {
                        await fs.writeFileSync(filePath, file.buffer);


                        const imagemRepository = getRepository(Imagens);
                        const imagem = imagemRepository.create({
                            path: path.relative(path.join(__dirname, '..', '..'), filePath),
                            viatura_id: result.id
                        });
                        await imagemRepository.save(imagem);
                        imagens.push(imagem);
                    } else {
                        console.log('Arquivo existente');
                    }


                }
            }

            return res.json(result);
        } catch (error) {
            return res.status(500).json("Erro ao salvar a viatura. Tente novamente.");
        }
    }

    static async delete(req: Request, res: Response): Promise<Response> {
        try {
            const { id } = req.params;
            const repository = getRepository(Viaturas);
            const result = await repository.softDelete(id);
            if (result.affected === 0) {
                return res.status(404).json('Viatura não encontrada');
            }
            return res.status(200).send();
        } catch (error) {
            return res.status(500).json("Erro ao excluir a viatura. Tente novamente.");
        }
    }

    static async buscarPorCodigo(req: Request, res: Response): Promise<Response> {
        try {
            const { userId } = req.body;
            const { codigo } = req.params;
            const repositoryUsuario = getRepository(User);

            const usuario = await repositoryUsuario.findOne({ where: { id: userId }, relations: ['unidades'] });
            if (!usuario) {
                return res.status(400).json('Usuário não encontrado, realize o login novamente.');
            }

            const unidadesIds = usuario.unidades.map(unidade => unidade.id);

            const codigoRegex = /^[A-Za-z]{1,4}-\d{1,4}$/;
            if (!codigoRegex.test(codigo)) {
                return res.status(400).json("Formato inválido para o código da viatura. O código deve ter o formato Ex.: AAA-123.");
            }

            const [prefixo, numero] = codigo.split('-');

            const repository = getRepository(Viaturas);
            const viatura = await repository.findOne({
                where: {
                    prefixo, numero, unidade: {
                        id: In(unidadesIds)
                    }
                }, relations: ["unidade", "imagens", "inventarios", "inventarios.tipoEquipamento", "inventarios.inventarioMateriais", "inventarios.inventarioMateriais.categoria", "inventarios.inventarioMateriais.equipamento"]
            });
            if (!viatura) {
                return res.status(400).json("Viatura não encontrada");
            }
            console.log('Viatura', viatura);
            return res.json(viatura);
        } catch (error) {
            console.error(error);
            return res.status(500).json(error.message);
        }
    }

    static async listarViaturas(req: Request, res: Response): Promise<Response> {
        const { unidade_id } = req.params;
        try {
            const repository = getRepository(Viaturas);

            const viaturas = await repository.find({
                where: {
                    unidade: {
                        id: unidade_id
                    }
                },
                order: { prefixo: 'ASC', numero: 'ASC' }
            });

            return res.json(viaturas);
        } catch (error) {
            return res.status(500).json(error.message);
        }
    }
}

export default ViaturaController;
